"""Analysis of the Rhine-water level data from https://pegel.bonn.de/php/rheinpegel.php ."""

from datetime import datetime

import matplotlib.pyplot as plt
import numpy as np
import pandas

from regularization import set_up_point_matrix

if __name__ == "__main__":
    rhein = pandas.read_csv("./data/pegel.tab", sep=" 	")
    levels = np.array([int(pegel.split(" ")[0]) for pegel in rhein["Pegel"]])

    timestamps = [ts[:-4] for ts in rhein["Zeit"]]
    datetime_list = []
    keep_indices = []
    for idx, ts in enumerate(timestamps):
        ts_date, ts_time = ts.split(",")
        day, month, year = ts_date.split(".")
        hour, minute = ts_time.split(":")
        datetime_list.append(datetime(int(year), int(month), int(day)))
        if hour == " 04" or hour == " 05":
            keep_indices.append(idx)

    # levels = levels[keep_indices]
    # datetime_list = np.array(datetime_list)[keep_indices]
    levels = levels[:-30_000:12]
    datetime_list = datetime_list[:-30_000:12]

    datetime_stamps = [dt.timestamp() for dt in datetime_list]

    levels = levels[::-1]
    datetime_stamps = datetime_stamps[::-1]
    datetime_list = datetime_list[::-1]

    plt.plot(datetime_stamps, levels)
    plt.show()

    point_no = len(levels)
    x_axis = np.array(datetime_stamps)
    A3 = set_up_point_matrix(x_axis, 2)

    xb = np.linalg.pinv(A3) @ levels
    est = x_axis * xb[1] + xb[0]

    plt.plot(datetime_list, levels)
    plt.plot(datetime_list, est)
    plt.xlabel("Year")
    plt.ylabel("Water level [cm]")
    plt.title("Rhine water level in Bonn")
    plt.show()

    zero = -xb[0] / xb[1]
    print("zero:", datetime.fromtimestamp(zero))

    x_axis = datetime_stamps / np.max(np.abs(datetime_stamps))
    degree = 20
    A4 = set_up_point_matrix(x_axis, degree=degree)

    xb = np.linalg.pinv(A4) @ levels
    est4 = A4 @ xb

    plt.plot(datetime_list, levels)
    plt.plot(datetime_list, est4)
    plt.title("Rhine water level in Bonn")
    plt.show()

    plt.plot(datetime_list, levels)

    U, sigma, V = np.linalg.svd(A4)

    for scale in [1e-1, 1e-4, 1e-9]:
        f_vec = sigma**2 / (sigma**2 + scale)
        F = np.diag(f_vec)
        S = np.zeros((degree, len(levels)))
        S[:degree, :degree] = np.diag(sigma ** (-1))
        x_reg = V.T @ F @ S @ U.T @ levels

        plt.plot(datetime_list, A4 @ x_reg, label=str(scale))
    plt.legend()
    plt.title("Rheinpegel bei Bonn")
    plt.ylabel("Pegel cm")
    plt.xlabel("Jahr")
    plt.show()
